/**   
 * @Title       : SafeManager.java
 * @Package     : com.ccb.common.safe
 * @Copyright   : 
 * @Description : E路护航相关加密方法
 * @author      : LiangYJ   
 * @date        : 2013-10-29
 * @version     : v1.0   
 */

package com.ccb.mobile.platform;

import com.ccb.mobile.platform.utils.MbsLogManager;

import CCB.NETBANK.SAFE.eSafeLib;
import android.content.Context;
import android.text.TextUtils;

/**
 * SafeManager
 * @author : LiangYJ
 * @date : 2013-10-29
 * 
 */
public class PlatformSafeManager {
	private static eSafeLib mESafeLib;
	
	/**
	 * 
	* 初始化方法，建议在Application的onCreate()方法里面初始化或者在程序的初始页面执行
	* @param context
	* @return void
	 */
	public static boolean init(final Context context){
		if(mESafeLib == null){
			mESafeLib = new eSafeLib(context);
		}
		boolean verify = mESafeLib.verify();
		MbsLogManager.logD("ESafeLogManager verify : " + verify);
		return verify;
	}
	
	public static String getEncryptString(String data){
		if(!TextUtils.isEmpty(data)){
			return eSafeLib.getEncryptString(data.getBytes());
		}
		return null;
	}
	
	public static String getVersion(){
		return eSafeLib.getVersion();
	}
	
	public static String getSignString(String messge){
		return mESafeLib.sign(messge);
	}
	
	//应用签名串3DES随机密钥加解密
	/**
	 * 
	* 3DES加密
	* @param plaintext
	* @return
	 */
	public static String get3DesEncryptString(String plaintext){
		return mESafeLib.get3DesEncryptString(plaintext);
	}
	
	/**
	 * 
	* 3DES解密
	* @param str 加密密文
	* @param keyStart 密钥起始位置
	* @param ivsStart 向量起始位置
	* @return 返回类型 String
	 */
	public static String get3DesDecryptString(String str,int keyStart,int ivsStart){
		return mESafeLib.get3DesDecrypt(str, keyStart, ivsStart);
	}
	
	/**
	 * 
	* 获取密钥起始位置,建议在执行get3DesEncryptString加密后执行此方法,
	* 				  以或许正确的密钥起始位置
	* @return 返回类型 int
	 */
	public static int get3DesKey(){
		return mESafeLib.get3DesKey();
	}
	
	/**
	 * 
	* 获取向量起始位置,建议在执行get3DesEncryptString加密后执行此方法,
	*                以获取正确的向量起始位置
	* @return 返回类型 int
	 */
	public static int get3Desivs(){
		return mESafeLib.get3Desivs();
	}
	
}
