/**   
 * @Package     : com.ccb.mobile.platform.db
 * @Copyright   : @版权所有 中国建设银行版权所有
 * @Description : 数据库操作工具类
 * @author      : dongguangyu   
 * @date        : 2014-4-24
 * @version     : v1.0   
 */

package com.ccb.mobile.platform.db;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;

import android.content.Context;
import android.content.SharedPreferences;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.util.Log;

import com.ccb.mobile.platform.bean.SYS_PACKAGE;
import com.ccb.mobile.platform.bean.SYS_PARAS;
import com.ccb.mobile.platform.bean.SYS_TXATTR;
import com.ccb.mobile.platform.utils.DESSecretTool;
import com.ccb.mobile.platform.utils.MbsLogManager;

public class DBUtil {
  private DBHelper dbHelper;
  private SQLiteDatabase db;
  private List<String> sqlList;

  public DBUtil(Context context, List<String> sqlList) {
    SharedPreferences preferences = context.getSharedPreferences("INITDB_TX", Context.MODE_PRIVATE);
    int db_version = preferences.getInt("PLATFORM_DB_VERSION", 1);
    MbsLogManager.logD("DBUtil(Context context,List<String> sqlListint,int db_version):::" + db_version);

    dbHelper = new DBHelper(context, db_version);
    db = dbHelper.getWritableDatabase();
    this.sqlList = sqlList;
  }

  public DBUtil(Context context) {

    SharedPreferences preferences = context.getSharedPreferences("INITDB_TX", Context.MODE_PRIVATE);
    int db_version = preferences.getInt("PLATFORM_DB_VERSION", 1);
    MbsLogManager.logD("DBUtil(Context context):::" + db_version);
    dbHelper = new DBHelper(context, db_version);
    db = dbHelper.getWritableDatabase();
  }

  // 加载系统参数信息
  public HashMap<String, String> iniSys_paras() {
    HashMap<String, String> paras_HashMap = new HashMap<String, String>();
    String sql = "select PARA_CODE, PARA_VALUE, PARA_DESC, CHANNEL_TYPE from SYS_PARAS";
    Cursor cursor = db.rawQuery(sql, null);

    if (cursor.getColumnCount() < 1) {
      return null;
    }

    while (cursor.moveToNext()) {
      try {
        paras_HashMap.put(cursor.getString(cursor.getColumnIndex("PARA_CODE")), DESSecretTool.decrypt(cursor.getString(cursor.getColumnIndex("PARA_VALUE"))));

      } catch (Exception e) {
        MbsLogManager.logD(e.getMessage());
        continue;
      }

    }
    cursor.close();
    return paras_HashMap;
  }

  // 加载系统参数对象信息
  public HashMap<String, SYS_PARAS> iniSysParams() {
    HashMap<String, SYS_PARAS> paras_HashMap = new HashMap<String, SYS_PARAS>();
    String sql = "select PARA_CODE, PARA_VALUE, PARA_DESC, CHANNEL_TYPE from SYS_PARAS";
    Cursor cursor = db.rawQuery(sql, null);

    if (cursor.getColumnCount() < 1) {
      return null;
    }
    SYS_PARAS syspara;
    while (cursor.moveToNext()) {

      try {
        String paraCode = cursor.getString(cursor.getColumnIndex("PARA_CODE"));
        String paraValue = DESSecretTool.decrypt(cursor.getString(cursor.getColumnIndex("PARA_VALUE")));
        String paraDesc = DESSecretTool.decrypt(cursor.getString(cursor.getColumnIndex("PARA_DESC")));
        String channelType = DESSecretTool.decrypt(cursor.getString(cursor.getColumnIndex("CHANNEL_TYPE")));
        syspara = new SYS_PARAS(paraCode, paraValue, paraDesc, channelType);
        paras_HashMap.put(paraCode, syspara);
      } catch (Exception e) {
        MbsLogManager.logD(e.getMessage());
        continue;
      }

    }
    cursor.close();
    return paras_HashMap;
  }

  // 初始化交易信息
  private HashMap<String, SYS_TXATTR> iniSys_txattr() {

    SYS_TXATTR tx = null;
    HashMap<String, SYS_TXATTR> sysTxs = new HashMap<String, SYS_TXATTR>();
    String sql = "select TXCODE,TXNAME,CHANNEL_TYPE,SUBMIT_TYPE,CHARACTER_ENCODING," + "PAKAGE_TYPE,URL,SNPAK,REVPAK ,ACTION_BEFORE,ACTION_AFTER from SYS_TXATTR order by TXCODE";

    Cursor cursor = db.rawQuery(sql, null);

    if (cursor.getColumnCount() < 1) {
      return null;
    }

    while (cursor.moveToNext()) {

      try {
        tx = new SYS_TXATTR();
        tx.setTxcode(cursor.getString(cursor.getColumnIndex("TXCODE")));
        tx.setTxName(DESSecretTool.decrypt(cursor.getString(cursor.getColumnIndex("TXNAME"))));
        tx.setChannelType(DESSecretTool.decrypt(cursor.getString(cursor.getColumnIndex("CHANNEL_TYPE"))));
        tx.setSubmitType(DESSecretTool.decrypt(cursor.getString(cursor.getColumnIndex("SUBMIT_TYPE"))));
        tx.setCharacterEncoding(DESSecretTool.decrypt(cursor.getString(cursor.getColumnIndex("CHARACTER_ENCODING"))));
        tx.setPakageType(DESSecretTool.decrypt(cursor.getString(cursor.getColumnIndex("PAKAGE_TYPE"))));
        tx.setUrl(DESSecretTool.decrypt(cursor.getString(cursor.getColumnIndex("URL"))));
        tx.setActionBefore(DESSecretTool.decrypt(cursor.getString(cursor.getColumnIndex("ACTION_BEFORE"))));
        tx.setActionAfter(DESSecretTool.decrypt(cursor.getString(cursor.getColumnIndex("ACTION_AFTER"))));
        tx.setSnPakid(DESSecretTool.decrypt(cursor.getString(cursor.getColumnIndex("SNPAK"))));
        tx.setRevPakid(DESSecretTool.decrypt(cursor.getString(cursor.getColumnIndex("REVPAK"))));
        sysTxs.put(cursor.getString(cursor.getColumnIndex("TXCODE")), tx);
      } catch (Exception e) {
        MbsLogManager.logD(e.getMessage());
        continue;
      }

    }
    cursor.close();
    return sysTxs;
  }

  private List<SYS_PACKAGE> iniSys_Package() {
    SYS_PACKAGE pak = null;
    List<SYS_PACKAGE> paks = new ArrayList<SYS_PACKAGE>();

    String sql = "select PAK_ID,TAG_ID,TAG_DES,REPLIST,MUSTIN,TAG_LENGTH from SYS_PACKAGE order by PAK_ID";

    Cursor cursor = db.rawQuery(sql, null);
    if (cursor.getColumnCount() < 1) {
      return null;
    }

    while (cursor.moveToNext()) {
      
      try {
        pak = new SYS_PACKAGE();
        pak.setPakId(cursor.getString(cursor.getColumnIndex("PAK_ID")));
        pak.setTagId(DESSecretTool.decrypt(cursor.getString(cursor.getColumnIndex("TAG_ID"))));
        pak.setTagDes(DESSecretTool.decrypt(cursor.getString(cursor.getColumnIndex("TAG_DES"))));
        pak.setRepList(DESSecretTool.decrypt(cursor.getString(cursor.getColumnIndex("REPLIST"))));
        pak.setMustIn(DESSecretTool.decrypt(cursor.getString(cursor.getColumnIndex("MUSTIN"))));
        pak.setTagLength(DESSecretTool.decrypt(cursor.getString(cursor.getColumnIndex("TAG_LENGTH"))));
        paks.add(pak);
      } catch (Exception e) {
        MbsLogManager.logD(e.getMessage());
        continue;
      }
      
    }
    cursor.close();
    return paks;
  }

  public HashMap<String, SYS_TXATTR> iniSys_txattrs() {

    HashMap<String, SYS_TXATTR> sysTxs = iniSys_txattr();
    List<SYS_PACKAGE> paks = iniSys_Package();

    HashMap<String, SYS_TXATTR> systxattr_HashMap = new HashMap<String, SYS_TXATTR>();
    Iterator<String> txIt = sysTxs.keySet().iterator();
    List<SYS_PACKAGE> snpaks;
    List<SYS_PACKAGE> revpaks;
    while (txIt.hasNext()) {
      String txcode = (String) txIt.next();
      SYS_TXATTR tx = sysTxs.get(txcode);
      String snpakid = tx.getSnPakid();
      String revpakid = tx.getRevPakid();

      if (snpakid != null || revpakid != null) {
        snpaks = new ArrayList<SYS_PACKAGE>();
        revpaks = new ArrayList<SYS_PACKAGE>();
        for (int i = 0; i < paks.size(); i++) {
          SYS_PACKAGE pak = paks.get(i);
          if (pak.getPakId().equals(snpakid)) {
            snpaks.add(pak);
          } else if (pak.getPakId().equals(revpakid)) {
            revpaks.add(pak);
          }
        }

        tx.setSnpaks(snpaks);
        tx.setRevpaks(revpaks);
      }
      systxattr_HashMap.put(txcode, tx);
    }

    return systxattr_HashMap;
  }

  public boolean initSystemDB() {
    int num = 0;
    boolean allExcu = true;
    try {
      db.beginTransaction();
      if (sqlList != null && !sqlList.isEmpty()) {
        for (String sql : sqlList) {
          if (checkSQL(sql) != -1) {
            num++;
            db.execSQL(encryptSQL(sql));
            // MbsLogManager.logD(num+"");
          } else {
            continue;
          }
        }
      }
      db.setTransactionSuccessful();
    } catch (Exception e) {
      allExcu = false;
      e.printStackTrace();
    } finally {
      db.endTransaction();
    }
    if (MbsLogManager.IS_DEBUG) {
      Log.d("DBUtil", "execued:" + num + "--all:" + sqlList.size());
    }
    return allExcu;
  }

  public static int checkSQL(String sql) {
    try {
      if (sql != null) {
        if (sql.trim().length() <= 0) {
          return -1;
        }

        String checkSql = sql.toUpperCase();

        int idx = 0;

        // 1.SQL语句禁止出现：奇数个单引号+破折号（--）
        while ((idx = checkSql.indexOf("--", idx)) > -1) {
          String strLeft = checkSql.substring(0, idx);
          if (strLeft.trim().length() <= 0 || !strLeft.trim().endsWith(";")) {
            MbsLogManager.logD(("SQL检查错误[check rule 1.0]！checkSql=[" + checkSql + "]"));
            return -1;
          }
          int iSingleQuotesCount = strLeft.length() - strLeft.replaceAll("'", "").length();
          if (iSingleQuotesCount % 2 == 1) {
            MbsLogManager.logD(("SQL检查错误[check rule 1]！checkSql=[" + checkSql + "]"));
            return -1;
          }
          idx += 2;
        }

        // 2.SQL语句禁止出现：块注释符（/**/）左边禁止出现奇数个单引号
        idx = -1;
        while ((idx = checkSql.indexOf("/*", idx)) > -1) {
          String strLeft = checkSql.substring(0, idx);
          if (strLeft.trim().length() <= 0) {
            MbsLogManager.logD(("SQL检查错误[check rule 2.0]！checkSql=[" + checkSql + "]"));
            return -1;
          }
          int iSingleQuotesCount = strLeft.length() - strLeft.replaceAll("'", "").length();
          if (iSingleQuotesCount % 2 == 1) {
            MbsLogManager.logD(("SQL检查错误[check rule 2.1]！checkSql=[" + checkSql + "]"));
            return -1;
          }
          idx += 2;
        }

        idx = -1;
        while ((idx = checkSql.indexOf("*/", idx)) > -1) {
          String strLeft = checkSql.substring(0, idx);
          int iSingleQuotesCount = strLeft.length() - strLeft.replaceAll("'", "").length();
          if (strLeft.trim().length() <= 0) {
            MbsLogManager.logD(("SQL检查错误[check rule 2.0]！checkSql=[" + checkSql + "]"));
            return -1;
          }
          if (iSingleQuotesCount % 2 == 1) {
            MbsLogManager.logD(("SQL检查错误[check rule 2.2]！checkSql=[" + checkSql + "]"));
            return -1;
          }
          idx += 2;
        }

        // 3.SQL语句禁止出现：未包含在单引号中的分号（;）
        idx = 0;
        while ((idx = checkSql.indexOf(";", idx)) > -1) {
          String strLeft = checkSql.substring(0, idx);
          String strRight = checkSql.substring(idx + 1);
          int iSingleQuotesCount1 = strLeft.length() - strLeft.replaceAll("'", "").length();
          int iSingleQuotesCount2 = strRight.length() - strRight.replaceAll("'", "").length();
          if (iSingleQuotesCount1 % 2 == 1 && iSingleQuotesCount2 % 2 == 1) {
            MbsLogManager.logD(("SQL检查错误[check rule 3]！checkSql=[" + checkSql + "]"));
            return -1;
          }
          idx += 1;
        }
        // 4.检查SQL语句合法性
        idx = 0;
        int iSingleQuotesCount_All = checkSql.length() - checkSql.replaceAll("'", "").length();
        if (iSingleQuotesCount_All % 2 == 1) {
          MbsLogManager.logD(("SQL检查错误[check rule 4]！checkSql=[" + checkSql + "]"));
          return -1;
        }
      }
    } catch (Exception e) {
      e.printStackTrace();
    }
    return 0;
  }

  public static String encryptSQL(String rawSql) {
    String str = rawSql;

    try {

      String[] rawSqlSplits = rawSql.split("\\(");

      if (rawSqlSplits != null && rawSqlSplits.length == 2) {

        int index = rawSqlSplits[1].lastIndexOf(")");
        if (index != -1 && index != 0) {

          String rawParams = rawSqlSplits[1].substring(0, index);

          String[] rawParamSplits = rawParams.split(",");
          if (rawParamSplits != null && rawParamSplits.length != 0) {

            str = rawSqlSplits[0] + "(";

            for (int i = 0; i < rawParamSplits.length; i++) {

              MbsLogManager.logD("encrypt params:"+rawParamSplits[i]);
              if (i == 0) {

                str = str + rawParamSplits[i] + ",";
              } else {

                if (rawParamSplits[i].contains("'")) {
                  rawParamSplits[i] = rawParamSplits[i].replace("'", "");
                }

                if ("".equals(rawParamSplits[i])) {
                  str = str + "'',";
                } else {
                  str = str + "'" + DESSecretTool.encrypt(rawParamSplits[i]) + "',";
                }

              }

            }

            str = str.substring(0, str.length() - 1);
            str += ")";
          }

        }

      }
    } catch (Exception e) {
      MbsLogManager.logD(e.getMessage());
    }

    MbsLogManager.logD("encrypt sql:" + str);

    return str;

  }
}
