package com.ccb.mobile.platform.http;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.Socket;
import java.net.SocketTimeoutException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URLEncoder;
import java.net.UnknownHostException;
import java.security.KeyManagementException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.zip.GZIPInputStream;

import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLHandshakeException;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;

import org.apache.http.Header;
import org.apache.http.HeaderElement;
import org.apache.http.HttpEntity;
import org.apache.http.HttpEntityEnclosingRequest;
import org.apache.http.HttpHost;
import org.apache.http.HttpRequest;
import org.apache.http.HttpResponse;
import org.apache.http.HttpResponseInterceptor;
import org.apache.http.HttpVersion;
import org.apache.http.NoHttpResponseException;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.CookieStore;
import org.apache.http.client.HttpRequestRetryHandler;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpDelete;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.client.params.ClientPNames;
import org.apache.http.client.params.HttpClientParams;
import org.apache.http.conn.ClientConnectionManager;
import org.apache.http.conn.params.ConnManagerParams;
import org.apache.http.conn.params.ConnRoutePNames;
import org.apache.http.conn.scheme.PlainSocketFactory;
import org.apache.http.conn.scheme.Scheme;
import org.apache.http.conn.scheme.SchemeRegistry;
import org.apache.http.conn.ssl.SSLSocketFactory;
import org.apache.http.cookie.Cookie;
import org.apache.http.cookie.CookieOrigin;
import org.apache.http.cookie.CookieSpec;
import org.apache.http.cookie.CookieSpecFactory;
import org.apache.http.cookie.MalformedCookieException;
import org.apache.http.entity.HttpEntityWrapper;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.conn.tsccm.ThreadSafeClientConnManager;
import org.apache.http.impl.cookie.BrowserCompatSpec;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.CoreProtocolPNames;
import org.apache.http.params.HttpConnectionParams;
import org.apache.http.params.HttpParams;
import org.apache.http.params.HttpProtocolParams;
import org.apache.http.protocol.BasicHttpContext;
import org.apache.http.protocol.ExecutionContext;
import org.apache.http.protocol.HTTP;
import org.apache.http.protocol.HttpContext;

import android.util.Log;

import com.ccb.mobile.platform.Constants;
import com.ccb.mobile.platform.system.BaseSystem;
import com.ccb.mobile.platform.utils.MbsLogManager;

/**
 * Wrap of org.apache.http.impl.client.DefaultHttpClient
 * 
 * @author LiangYJ
 * 
 */
public class HttpClient {

  private static final String TAG = "CCB_HttpClient";
  // private final static boolean DEBUG = true;

  /** OK: Success! */
  public static final int OK = 200;
  /** Not Modified: There was no new data to return. */
  public static final int NOT_MODIFIED = 304;
  /**
   * Bad Request: The request was invalid. An accompanying error message will explain why. This is the status code will be returned during rate limiting.
   */
  public static final int BAD_REQUEST = 400;
  /** Not Authorized: Authentication credentials were missing or incorrect. */
  public static final int NOT_AUTHORIZED = 401;
  /**
   * Forbidden: The request is understood, but it has been refused. An accompanying error message will explain why.
   */
  public static final int FORBIDDEN = 403;
  /**
   * Not Found: The URI requested is invalid or the resource requested, such as a user, does not exists.
   */
  public static final int NOT_FOUND = 404;
  /**
   * Not Acceptable: Returned by the Search API when an invalid format is specified in the request.
   */
  public static final int NOT_ACCEPTABLE = 406;
  /**
   * Internal Server Error: Something is broken. Please post to the group so the Weibo team can investigate.
   */
  public static final int INTERNAL_SERVER_ERROR = 500;
  /** Bad Gateway: Weibo is down or being upgraded. */
  public static final int BAD_GATEWAY = 502;
  /**
   * Service Unavailable: The Weibo servers are up, but overloaded with requests. Try again later. The search and trend methods use this to indicate when you are being rate limited.
   */
  public static final int SERVICE_UNAVAILABLE = 503;
  public static final int TIME_OUT = 504;
  public static final int ACCESS_DENIED = 407;
  public static final int Partial_Content = 206;

  private static final int CONNECTION_TIMEOUT_MS = 45 * 1000;
  private static final int SOCKET_TIMEOUT_MS = 45 * 1000;

  public static final int RETRIEVE_LIMIT = 20;
  public static final int RETRIED_TIME = 3;

  // private static final String SERVER_HOST = "api.fanfou.com";

  private DefaultHttpClient mClient;
  private BasicHttpContext localcontext;

  private CookieStore mCookieStore;
  private String mCookieString;
  private static boolean isAuthenticationEnabled = false;

  private static final String USER_AGENT = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2)";

  // CHART-SET
  public static final String UTF_8 = "UTF-8";
  public static final String GB2312 = "GB2312";
  public static final String GBK = "GBK";
  public static final String ISO_8859_1 = "ISO-8859-1";

  private String chart_set;
  private HashMap<String, String> headers;

  public HttpClient() {
    prepareHttpClient();
  }

  /**
   * 
   * @Title : setChartSet
   * @Description : 常量 UTF_8,GB2312,GBK或者其他自定义字符集
   * @param : @param chartSet 设定文件
   * @return : void 返回类型
   * @throws
   */
  public void setChartSet(String chartSet) {
    this.chart_set = chartSet;
  }

  public void setHeaders(HashMap<String, String> headers) {
    this.headers = headers;
  }

  // cookie管理
  public CookieStore getCookie() {
    return mCookieStore;
  }

  public void setCookie(CookieStore cookieStore) {
    mCookieStore = cookieStore;
  }

  public void setCookieStr(String cooString) {
    mCookieString = cooString;
  }

  public String getCookieStr() {
    return mCookieString;
  }

  /**
   * @param hostname
   *          the hostname (IP or DNS name)
   * @param port
   *          the port number. -1 indicates the scheme default port.
   * @param scheme
   *          the name of the scheme. null indicates the default scheme
   */
  public void setProxy(String host, int port, String scheme) {
    HttpHost proxy = new HttpHost(host, port, scheme);
    mClient.getParams().setParameter(ConnRoutePNames.DEFAULT_PROXY, proxy);
  }

  public void removeProxy() {
    mClient.getParams().removeParameter(ConnRoutePNames.DEFAULT_PROXY);
  }

  /**
   * Setup DefaultHttpClient
   * 
   * Use ThreadSafeClientConnManager.
   * 
   */
  private void prepareHttpClient2() {
    // Create and initialize HTTP parameters
    HttpParams params = new BasicHttpParams();
    ConnManagerParams.setMaxTotalConnections(params, 10);
    HttpProtocolParams.setVersion(params, HttpVersion.HTTP_1_1);
    KeyStore trustStore;
    try {
      trustStore = KeyStore.getInstance(KeyStore.getDefaultType());
      trustStore.load(null, null);
      SSLSocketFactory sf = new SSLSocketFactoryEx(trustStore);
      sf.setHostnameVerifier(SSLSocketFactory.ALLOW_ALL_HOSTNAME_VERIFIER); // 允许所有主机的验证
      HttpProtocolParams.setVersion(params, HttpVersion.HTTP_1_1);
      HttpProtocolParams.setContentCharset(params, HTTP.DEFAULT_CONTENT_CHARSET);
      HttpProtocolParams.setUseExpectContinue(params, true);

      // Create and initialize scheme registry
      SchemeRegistry schemeRegistry = new SchemeRegistry();
      schemeRegistry.register(new Scheme("http", PlainSocketFactory.getSocketFactory(), 80));
      // schemeRegistry.register(new Scheme("https", SSLSocketFactory.getSocketFactory(), 443)); // 生产
      schemeRegistry.register(new Scheme("https", sf, 443)); // 测试
      ThreadSafeClientConnManager cm = new ThreadSafeClientConnManager(params, schemeRegistry);
      mClient = new DefaultHttpClient(cm, params);

    } catch (KeyStoreException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    } catch (NoSuchAlgorithmException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    } catch (CertificateException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    } catch (IOException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    } catch (KeyManagementException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    } catch (UnrecoverableKeyException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }

    // Create an HttpClient with the ThreadSafeClientConnManager.
    // Support GZIP
    mClient.addResponseInterceptor(gzipResponseIntercepter);

    mClient.getParams().setParameter(CoreProtocolPNames.USER_AGENT, USER_AGENT);
    mClient.getParams().setParameter(CoreProtocolPNames.USE_EXPECT_CONTINUE, Boolean.FALSE);
    // if(MbsLogManager.IS_DEBUG){
    // Log.d(TAG, "request HTTP_CONTENT_CHARSET:"+chart_set);
    // }
    // mClient.getParams().setParameter(CoreProtocolPNames.HTTP_CONTENT_CHARSET, chart_set);
    mClient.getCookieSpecs().register("UNCHECK_DOMAIN", myCookiePolice);
    mClient.getParams().setParameter(ClientPNames.COOKIE_POLICY, "UNCHECK_DOMAIN");
    // mClient.getParams().setParameter(ClientPNames.COOKIE_POLICY ,CookiePolicy.BROWSER_COMPATIBILITY);
  }

  private void prepareHttpClient() {
    try {
      // 设置参数列表对象
      HttpParams params = new BasicHttpParams();
      ConnManagerParams.setMaxTotalConnections(params, 10);
      // 设置http协议版本
      HttpProtocolParams.setVersion(params, HttpVersion.HTTP_1_1);
      // 设置编码类型
      HttpProtocolParams.setContentCharset(params, HTTP.UTF_8);
      // 设置异常是否继续???
      HttpProtocolParams.setUseExpectContinue(params, true);
      HttpProtocolParams.setHttpElementCharset(params, HTTP.UTF_8);
      // 设置客户端参数
      HttpClientParams.setRedirecting(params, true);// 设置中是否重定向
      HttpClientParams.setAuthenticating(params, false);
      // 设置超时
      HttpConnectionParams.setStaleCheckingEnabled(params, true);
      HttpConnectionParams.setConnectionTimeout(params, CONNECTION_TIMEOUT_MS);// 设置链接超时时间
      HttpConnectionParams.setSoTimeout(params, SOCKET_TIMEOUT_MS);// 设置socket超时时间
      HttpConnectionParams.setSocketBufferSize(params, 512 * 1024);// 设置sokect缓存最大字节数
      ConnManagerParams.setTimeout(params, CONNECTION_TIMEOUT_MS);// 从连接池获取连接超时时间
      // 注册通讯http和https模式对应的socket工厂对象
      SchemeRegistry registry = new SchemeRegistry();
      // 注册http模式，普通socket
      registry.register(new Scheme("http", PlainSocketFactory.getSocketFactory(), 80));
      // 注册https模式，SSLsoket
      if (MbsLogManager.CLOSE_VERIFY_CERTIFICATE) {
        // 生成证书，账号密码都为null
        KeyStore trustStore = KeyStore.getInstance(KeyStore.getDefaultType());
        trustStore.load(null, null);
        // 生成自定义的socket工厂对象
        SSLSocketFactory sf = new BTCSSLSocketFactory(trustStore);
        // 设置信任主机
        sf.setHostnameVerifier(SSLSocketFactory.ALLOW_ALL_HOSTNAME_VERIFIER);
        // ===测试环境不验证书
        registry.register(new Scheme("https", sf, 443));
      } else {
        // ===生产环境，校验证书
        registry.register(new Scheme("https", SSLSocketFactory.getSocketFactory(), 443));
      }
      // 生成链接管理对象，把设置的参数和模式设置进去
      ClientConnectionManager ccm = new ThreadSafeClientConnManager(params, registry);
      // 返回HttpClient对象
      mClient = new DefaultHttpClient(ccm, params);
    } catch (Exception e) {
      MbsLogManager.logE(e.toString());
      mClient = new DefaultHttpClient();
    }
    
    mClient.addResponseInterceptor(gzipResponseIntercepter);
    mClient.getParams().setParameter(CoreProtocolPNames.USER_AGENT, USER_AGENT);
    mClient.getParams().setParameter(CoreProtocolPNames.USE_EXPECT_CONTINUE, Boolean.FALSE);

    HttpConnectionParams.setConnectionTimeout(mClient.getParams(), CONNECTION_TIMEOUT_MS);// 设置链接超时时间
    HttpConnectionParams.setSoTimeout(mClient.getParams(), SOCKET_TIMEOUT_MS);// 设置socket超时时间
  }

  // 使用单例模式，导致多个系统切换调用没有设置编码
  public void setHTTP_CONTENT_CHARSET() {
    if (MbsLogManager.IS_DEBUG) {
      Log.d(TAG, "request HTTP_CONTENT_CHARSET:" + chart_set);
    }
    mClient.getParams().setParameter(CoreProtocolPNames.HTTP_CONTENT_CHARSET, chart_set);
  }

  private static HttpResponseInterceptor gzipResponseIntercepter = new HttpResponseInterceptor() {

    @Override
    public void process(HttpResponse response, HttpContext context) throws org.apache.http.HttpException, IOException {
      HttpEntity entity = response.getEntity();
      Header ceheader = entity.getContentEncoding();
      if (ceheader != null) {
        HeaderElement[] codecs = ceheader.getElements();
        for (int i = 0; i < codecs.length; i++) {
          if (codecs[i].getName().equalsIgnoreCase("gzip")) {
            response.setEntity(new GzipDecompressingEntity(response.getEntity()));
            return;
          }
        }
      }

    }
  };

  static class GzipDecompressingEntity extends HttpEntityWrapper {

    public GzipDecompressingEntity(final HttpEntity entity) {
      super(entity);
    }

    @Override
    public InputStream getContent() throws IOException, IllegalStateException {

      // the wrapped entity's getContent() decides about repeatability
      InputStream wrappedin = wrappedEntity.getContent();
      return new GZIPInputStream(wrappedin);
    }

    @Override
    public long getContentLength() {
      // length of ungzipped content is not known
      return -1;
    }

  }

  public Response post(String url, ArrayList<BasicNameValuePair> postParams) throws HttpException {
    if (null == postParams) {
      postParams = new ArrayList<BasicNameValuePair>();
    }
    return httpRequest(url, postParams, HttpPost.METHOD_NAME);
  }

  public Response post(String url) throws HttpException {
    return httpRequest(url, null, HttpPost.METHOD_NAME);
  }

  public Response post(String url, HashMap<String, String> postmap) throws HttpException {
    return httpRequest(url, getBasicNameValuePairs(postmap), HttpPost.METHOD_NAME);
  }

  /**
   * POST
   * 
   * @param url
   * @param file
   * @param authenticate
   * @return
   * @throws HttpException
   */
  public Response post(String url, File file) throws HttpException {
    return httpRequest(url, null, file, HttpPost.METHOD_NAME);
  }

  public Response get(String url, HashMap<String, String> getmap) throws HttpException {
    return httpRequest(url, getBasicNameValuePairs(getmap), HttpGet.METHOD_NAME);
  }

  public Response get(String url, ArrayList<BasicNameValuePair> params, boolean authenticated) throws HttpException {
    return httpRequest(url, params, HttpGet.METHOD_NAME);
  }

  public Response get(String url, ArrayList<BasicNameValuePair> params) throws HttpException {
    return httpRequest(url, params, HttpGet.METHOD_NAME);
  }

  public Response get(String url) throws HttpException {
    return httpRequest(url, null, HttpGet.METHOD_NAME);
  }

  public Response httpRequest(String url, ArrayList<BasicNameValuePair> postParams, String httpMethod) throws HttpException {
    return httpRequest(url, postParams, null, httpMethod);
  }

  /**
   * Execute the DefaultHttpClient
   * 
   * @param url
   *          target
   * @param postParams
   * @param file
   *          can be NULL
   * @param authenticated
   *          need or not
   * @param httpMethod
   *          HttpPost.METHOD_NAME HttpGet.METHOD_NAME HttpDelete.METHOD_NAME
   * @return Response from server
   * @throws HttpException
   *           此异常包装了多列底层异常<br />
   * <br />
   *           1. 底层异常, 可使用getCause()查看: <br />
   *           <li>URISyntaxException, 由`new URI` 引发</li> <li>IOException, 由`createMultipartEntity`、`UrlEncodedFormEntity` 引发</li> <li>IOException和ClientProtocolException, 由`HttpClient.execute` 引发</li>
   * <br />
   * 
   *           2. 当响应码不为200时报出的各种子类异常: <li>HttpRequestException, 通常发生在请求的错误,如请求错误了 网址导致404抛出此异 首先request log, 确认不是人为错误导致请求失败</li> <li>HttpAuthException, 通常发生在Auth失败, 用于验证登录的用户名/密码/KEY/li> <li>
   *           HttpRefusedException, 通常发生在服务器接受到请求 但拒绝请求 可是多种原因, 具体原因 服务器会返回拒绝理由, 调用HttpRefusedException#getError#getMessage查看</li> <li>
   *           HttpServerException, 通常发生在服务器发生错误 服务器端是否在正常提供服</li> <li>
   *           HttpException, 其他未知错误.</li>
   */
  public Response httpRequest(String url, ArrayList<BasicNameValuePair> postParams, File file, String httpMethod) throws HttpException {
    if (MbsLogManager.IS_DEBUG) {
      Log.d(TAG, "Sending " + httpMethod + " request to " + url);
    }
    URI uri = createURI(url);

    HttpResponse response = null;
    Response res = null;
    HttpUriRequest method = null;

    // Create POST, GET or DELETE METHOD
    method = createMethod(httpMethod, uri, file, postParams);
    // Setup ConnectionParams, Request Headers
    SetupHTTPConnectionParams(method);
    // Execute Request
    try {
      if (mCookieStore != null) {
        mClient.setCookieStore(mCookieStore);
      } else {
        method.addHeader("Cookie", mCookieString);
      }
      // 自定义的Header参数不为空，设置Header
      if (headers != null && !headers.isEmpty()) {
        for (String key : headers.keySet()) {
          method.addHeader(key, headers.get(key));
        }
      }
      if (MbsLogManager.IS_DEBUG) {
        Header[] requestHeaders = method.getAllHeaders();
        Log.v(TAG, "Request Headers : ");
        for (Header header : requestHeaders) {
          Log.v(TAG, header.getName() + " = " + header.getValue());
        }
        if (mCookieStore != null) {
          for (Cookie cookie : mClient.getCookieStore().getCookies()) {
            Log.d(TAG, "Cookie：" + cookie.getName() + " = " + cookie.getValue());
          }
        }

      }
      response = mClient.execute(method, localcontext);
      mCookieStore = mClient.getCookieStore();
      Log.d(TAG, "response.StatusCode:"+response.getStatusLine().getStatusCode());
      // 保存cookie
      // mCookieString = saveCookieForHttp(response);
      // setCookieStr(mCookieString);
      Header[] responseHeaders = response.getAllHeaders();
      BaseSystem.setResponseHeaders(responseHeaders);
//      if (MbsLogManager.IS_DEBUG) {
//        Log.d(TAG, "Response Headers : ");
//        for (Header header : responseHeaders) {
//          Log.d(TAG, header.getName() + " = " + header.getValue());
//        }
//      }
      res = new Response(response);
      MbsLogManager.logD("========response================");
      for (Header header : responseHeaders) {
    	  if (MbsLogManager.IS_DEBUG)
    	  Log.d(TAG, header.getName() + " = " + header.getValue());
    	  if("Accept-Ranges".equalsIgnoreCase(header.getName())){
    		  res.setmAccept_Ranges(header.getValue());
    	  }else if("Content-Range".equalsIgnoreCase(header.getName())){
    		  res.setmContent_Range(header.getValue());
    	  }
        }
    } catch (ClientProtocolException e) {
       Log.e(TAG, e.getMessage(), e);
      throw new HttpException(e.getMessage(), e, NOT_FOUND);
    } catch (SocketTimeoutException e) {
      Log.e(TAG, e.getMessage(), e);
      throw new HttpException(e.getMessage(), e, TIME_OUT);
    } catch (IOException ioe) {
      Log.e(TAG, ioe.getMessage(), ioe);
      throw new HttpException(ioe.getMessage(), ioe, FORBIDDEN);
    }

    if (response != null) {
      int statusCode = response.getStatusLine().getStatusCode();
      Log.d(TAG, "statusCode---------------"+statusCode);
      // It will throw a weiboException while status code is not 200
      HandleResponseStatusCode(statusCode, res);
    } else {
       Log.e(TAG, "response is null");
    }

    return res;
  }

  /**
   * CreateURI from URL string
   * 
   * @param url
   * @return request URI
   * @throws HttpException
   *           Cause by URISyntaxException
   */
  private URI createURI(String url) throws HttpException {
    URI uri;

    try {
      uri = new URI(url);
    } catch (URISyntaxException e) {
      if (MbsLogManager.IS_DEBUG) {
        Log.e(TAG, e.getMessage(), e);
      }
      throw new HttpException("Invalid URL.");
    }

    return uri;
  }

  /**
   * 
   * This isHttpClient.java class
   * 
   * @param HttpUriRequest
   *          method
   * @return void: 返回类型
   * 
   */
  private void SetupHTTPConnectionParams(HttpUriRequest method) {
    HttpConnectionParams.setConnectionTimeout(method.getParams(), CONNECTION_TIMEOUT_MS);
    HttpConnectionParams.setSoTimeout(method.getParams(), SOCKET_TIMEOUT_MS);
    mClient.setHttpRequestRetryHandler(requestRetryHandler);
    method.addHeader("Accept-Encoding", "gzip, deflate");
    method.addHeader("Accept-Charset", "UTF-8,*;q=0.5");

  }

  /**
   * Create request method, such as POST, GET, DELETE
   * 
   * @param httpMethod
   *          "GET","POST","DELETE"
   * @param uri
   *          请求的URI
   * @param file
   *          可为null
   * @param postParams
   *          POST参数
   * @return httpMethod Request implementations for the various HTTP methods like GET and POST.
   * @throws HttpException
   *           createMultipartEntity UrlEncodedFormEntity引发的IOException
   */
  private HttpUriRequest createMethod(String httpMethod, URI uri, File file, ArrayList<BasicNameValuePair> postParams) throws HttpException {

    HttpUriRequest method;

    if (httpMethod.equalsIgnoreCase(HttpPost.METHOD_NAME)) {
      // POST METHOD

      HttpPost post = new HttpPost(uri);
      // See this:
      // http://groups.google.com/group/twitter-development-talk/browse_thread/thread/e178b1d3d63d8e3b
      post.getParams().setBooleanParameter("http.protocol.expect-continue", false);

      try {
        HttpEntity entity = null;
        if (null != file) {
        } else if (null != postParams) {
          entity = new UrlEncodedFormEntity(postParams, chart_set);
        }
        post.setEntity(entity);
      } catch (IOException ioe) {
        throw new HttpException(ioe.getMessage(), ioe);
      }

      method = post;
    } else if (httpMethod.equalsIgnoreCase(HttpDelete.METHOD_NAME)) {
      method = new HttpDelete(uri);
    } else {
      method = new HttpGet(uri);
    }

    return method;
  }

  /**
   * 解析HTTP错误
   * 
   * @param statusCode
   * @return
   */
  private static String getCause(int statusCode) {
    String cause = null;
    switch (statusCode) {
    case NOT_MODIFIED:
      break;
    case BAD_REQUEST:
      cause = "The request was invalid.  An accompanying error message will explain why. This is the status code will be returned during rate limiting.";
      break;
    case NOT_AUTHORIZED:
      cause = "Authentication credentials were missing or incorrect.";
      break;
    case FORBIDDEN:
      cause = "The request is understood, but it has been refused.  An accompanying error message will explain why.";
      break;
    case NOT_FOUND:
      cause = "The URI requested is invalid or the resource requested, such as a user, does not exists.";
      break;
    case NOT_ACCEPTABLE:
      cause = "Returned by the Search API when an invalid format is specified in the request.";
      break;
    case INTERNAL_SERVER_ERROR:
      cause = "Something is broken.  Please post to the group so the Weibo team can investigate.";
      break;
    case BAD_GATEWAY:
      cause = "Weibo is down or being upgraded.";
      break;
    case SERVICE_UNAVAILABLE:
      cause = "Service Unavailable: The Weibo servers are up, but overloaded with requests. Try again later. The search and trend methods use this to indicate when you are being rate limited.";
      break;
//    case Partial_Content:
//    	cause = "partial content";
//    	break;
    default:
      cause = "";
    }
    return statusCode + ":" + cause;
  }

  public boolean isAuthenticationEnabled() {
    return isAuthenticationEnabled;
  }

  public static void log(String msg) {
    if (MbsLogManager.IS_DEBUG) {
      Log.d(TAG, msg);
    }
  }

  /**
   * Handle Status code
   * 
   * @param statusCode
   *          响应的状态码
   * @param res
   *          服务器响信息
   * @throws HttpException
   *           当响应码不为200时都会报出此异常:<br />
   *           <li>HttpRequestException, 通常发生在请求的错误,如请求错误了 网址导致404发生 抛出此异常 首先request log, 确认不是人为错误导致请求失败</li> <li>HttpAuthException, 通常发生在Auth失败, 如果用于验证登录的用户名/密码/KEY</li> <li>
   *           HttpRefusedException, 通常发生在服务器接受到请求 但拒绝请求 可是多种原因, 具体原因 服务器会返回拒绝理由, 调用HttpRefusedException#getError#getMessage查看</li> <li>HttpServerException, 通常发生在服务器发生错误，服务器端是否在正常提供服</li> <li>
   *           HttpException, 其他未知错误.</li>
   */
  private void HandleResponseStatusCode(int statusCode, Response res) throws HttpException {
    String msg = getCause(statusCode) + "\n";
    switch (statusCode) {
    // It's OK, do nothing
    case OK:
      break;

    // Mine mistake, Check the Log
    case NOT_MODIFIED:
    case BAD_REQUEST:
    case NOT_FOUND:
    case NOT_ACCEPTABLE:
    case ACCESS_DENIED:
      throw new HttpException(msg + res.asString(chart_set, false), statusCode);

      // UserName/Password incorrect
    case NOT_AUTHORIZED:
      throw new HttpAuthException(msg + res.asString(chart_set, false), statusCode);

      // Server will return a error message, use
      // HttpRefusedException#getError() to see.
    case FORBIDDEN:
      throw new HttpRefusedException(msg, statusCode);

      // Something wrong with server
    case INTERNAL_SERVER_ERROR:
    case BAD_GATEWAY:
    case SERVICE_UNAVAILABLE:
      throw new HttpServerException(msg, statusCode);

    case Partial_Content:
    	break;
      // Others
    default:
      throw new HttpException(msg + res.asString(chart_set, false), statusCode);
    }
  }

  public static String encode(String value) throws HttpException {
    try {
      return URLEncoder.encode(value, HTTP.UTF_8);
    } catch (UnsupportedEncodingException e_e) {
      throw new HttpException(e_e.getMessage(), e_e);
    }
  }

  public static String encodeParameters(ArrayList<BasicNameValuePair> params) throws HttpException {
    StringBuffer buf = new StringBuffer();
    for (int j = 0; j < params.size(); j++) {
      if (j != 0) {
        buf.append("&");
      }
      try {
        buf.append(URLEncoder.encode(params.get(j).getName(), "UTF-8")).append("=").append(URLEncoder.encode(params.get(j).getValue(), "UTF-8"));
      } catch (java.io.UnsupportedEncodingException neverHappen) {
        throw new HttpException(neverHappen.getMessage(), neverHappen);
      }
    }
    return buf.toString();
  }

  /**
   * 异常自动恢复处理, 使用HttpRequestRetryHandler接口实现请求的异常恢复
   */
  private static HttpRequestRetryHandler requestRetryHandler = new HttpRequestRetryHandler() {
    // 自定义的恢复策略
    public boolean retryRequest(IOException exception, int executionCount, HttpContext context) {
      // 设置恢复策略，在发生异常时候将自动重试
      if (executionCount >= RETRIED_TIME) {
        // Do not retry if over max retry count
        return false;
      }
      if (exception instanceof NoHttpResponseException) {
        // Retry if the server dropped connection on us
        return true;
      }
      if (exception instanceof SSLHandshakeException) {
        // Do not retry on SSL handshake exception
        return false;
      }
      HttpRequest request = (HttpRequest) context.getAttribute(ExecutionContext.HTTP_REQUEST);
      boolean idempotent = (request instanceof HttpEntityEnclosingRequest);
      if (!idempotent) {
        // Retry if the request is considered idempotent
        return true;
      }
      return false;
    }
  };

  class SSLSocketFactoryEx extends SSLSocketFactory {

    SSLContext sslContext = SSLContext.getInstance("TLS");

    public SSLSocketFactoryEx(KeyStore truststore) throws NoSuchAlgorithmException, KeyManagementException, KeyStoreException, UnrecoverableKeyException {
      super(truststore);

      TrustManager tm = new X509TrustManager() {

        @Override
        public java.security.cert.X509Certificate[] getAcceptedIssuers() {
          return null;
        }

        @Override
        public void checkClientTrusted(java.security.cert.X509Certificate[] chain, String authType) throws java.security.cert.CertificateException {

        }

        @Override
        public void checkServerTrusted(java.security.cert.X509Certificate[] chain, String authType) throws java.security.cert.CertificateException {

        }
      };

      sslContext.init(null, new TrustManager[] { tm }, null);
    }

    @Override
    public Socket createSocket(Socket socket, String host, int port, boolean autoClose) throws IOException, UnknownHostException {
      return sslContext.getSocketFactory().createSocket(socket, host, port, autoClose);
    }

    @Override
    public Socket createSocket() throws IOException {
      return sslContext.getSocketFactory().createSocket();
    }
  }

  /**
   * 
   * 将参数从HashMap<String, String>类型转换成ArrayList<BasicNameValuePair>类型，以便进行网络操作
   */
  private ArrayList<BasicNameValuePair> getBasicNameValuePairs(HashMap<String, String> params) {
    ArrayList<BasicNameValuePair> list = null;
    if (params != null && !params.isEmpty()) {
      list = new ArrayList<BasicNameValuePair>(params.size());
      for (String key : params.keySet()) {
        if (MbsLogManager.IS_DEBUG) {
          Log.d(TAG, key + "--" + params.get(key));
        }
        list.add(new BasicNameValuePair(key, params.get(key)));

      }
    }
    return list;
  }

  /**
   * 方法功能说明：从通讯结果中保存cookie
   * 
   * @param HttpResponse
   * 
   */
  public String saveCookieForHttp(HttpResponse response) {
    if (response == null) {
      return null;
    }
    Header[] headers = response.getAllHeaders();
    for (int i = 0, len = headers.length; i < len; i++) {
      Header header = headers[i];
      String name = header.getName();
      String value = header.getValue();
      if (Constants.SET_COOKIE.equals(name)) {
        int index = value.indexOf("=");
        String cookieName = "";
        if (index != -1) {
          cookieName = value.substring(0, index);

          if (Constants.COOKIE_NAME.equalsIgnoreCase(cookieName)) {
            mCookieString = value;
          }
        }
      }
    }

    if (mCookieString.endsWith(";")) {
      mCookieString = mCookieString.substring(0, mCookieString.length() - 1);
    }
    Log.e(TAG, "Cookie：" + mCookieString);
    return mCookieString;
  }

  // 通过自定义的方式，解决了cookie reject的问题。
  // http://hc.apache.org/httpcomponents-client-ga/tutorial/html/statemgmt.html#d5e799
  CookieSpecFactory myCookiePolice = new CookieSpecFactory() {
    public CookieSpec newInstance(HttpParams params) {
      return new BrowserCompatSpec() {
        @Override
        public void validate(Cookie cookie, CookieOrigin origin) throws MalformedCookieException {
          // Oh, I am easy
        }
      };
    }
  };

}
